
/* This file was generated automatically by the Snowball to ANSI C compiler */

#ifdef __cplusplus
extern "C" {
#endif

extern struct SN_env * porter_ISO_8859_1_create_env(void);
extern void porter_ISO_8859_1_close_env(struct SN_env * z);

extern int porter_ISO_8859_1_stem(struct SN_env * z);

#ifdef __cplusplus
}
#endif

